@@ -1,7 +1,7 @@
 /*=============================================================================
 ttyrpld - TTY replay daemon
 user/rdsh.c - Shared functions for RPLD/INFOD/RPLCTL
-  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
+  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
   -- License restrictions apply (GPL2)
 
   This file is part of ttyrpld.
@@ -186,7 +186,8 @@ void log_close(struct tty *tty) {
     if(tty->status != IFP_DEACTIVATE) {
         /* If the status is IFP_ACTIVATED, it is reinstantiated upon next
         get_tty(). If it is IFP_DEACTIVSES, it will change to IFP_ACTIVATED,
-        as per definition. Only keep IFP_DEACTIVATED. */
+        as per definition. So we only need the data structure if
+        IFP_DEACTIVATED is on. */
         HXbtree_del(Ttys, (void *)tty->dev);
         free(tty);
     }
