@@ -1,7 +1,7 @@
 /*=============================================================================
 ttyrpld - TTY replay daemon
 user/rplctl.c - INFOD client
-  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
+  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
   -- License restrictions apply (GPL2)
 
   This file is part of ttyrpld.
@@ -38,107 +38,43 @@ user/rplctl.c - INFOD client
 #include "global.h"
 #include "rdsh.h"
 
+#define IFP_GETINFO_A (Opt.parseable ? IFP_GETINFO : IFP_GETINFO_T)
+
 // Function prototypes
-static void mainloop(int);
 static uint32_t getdev(const char *);
+static void send_int(int, unsigned char, uint32_t);
+static void send_void(int, unsigned char);
+static void send_string(int, unsigned char, const char *);
 static int unix_client(const char *);
 
 static int get_options(int *, const char ***);
 
 // Global variables
-static struct HXlld *Jobs = NULL;
 static struct {
     int do_listing, parseable;
 } Opt = {
     .do_listing = 0, // -L
     .parseable  = 0, // -t
 };
+static int Sockfd = -1;
 
 //-----------------------------------------------------------------------------
 int rplctl_main(int argc, const char **argv) {
-    int fd;
-
-    if((Jobs = HXlld_init(0)) == NULL) {
-        perror("malloc()");
-        return EXIT_FAILURE;
-    }
-
-    if(!get_options(&argc, &argv)) { return EXIT_FAILURE; }
-
-    if((fd = unix_client(GOpt.infod_port)) < 0) {
+    if((Sockfd = unix_client(GOpt.infod_port)) < 0) {
         fprintf(stderr, "connect() to %s failed: %s\n",
          GOpt.infod_port, strerror(errno));
         return EXIT_FAILURE;
     }
 
-    if(Jobs->itemcount == 0 && argc < 1) {
-        /* If no arguments are given at all (or -L is given explicitly), get
-        information about all ttys and INFOD. */
-        HXlld_push(Jobs, (void *)(Opt.parseable ? IFP_GETINFO : IFP_GETINFO_T));
-        HXlld_push(Jobs, NULL);
+    if(get_options(&argc, &argv) == 0) {
+        send_int(Sockfd, IFP_GETINFO_A, 0);
     }
 
-    while(*argv != NULL) {
-        uint32_t dev;
-        if((dev = getdev(*argv)) != 0) {
-            HXlld_push(Jobs, (void *)(Opt.parseable ? IFP_GETINFO : IFP_GETINFO_T));
-            HXlld_push(Jobs, (void *)dev);
-        }
-        ++argv;
-    }
-
-    mainloop(fd);
-    close(fd);
-    HXlld_free(Jobs);
+    send_int(Sockfd, IFP_NONE, 0);
+    close(Sockfd);
     return EXIT_SUCCESS;
 }
 
-static void mainloop(int fd) {
-    unsigned char req;
-    char buf[4096];
-
-    while((req = (unsigned char)(int)HXlld_shift(Jobs)) != 0) {
-        /* Protocol:
-        1. rplctl sends request type (1 byte)
-        (1a. infod connection ends if request is IFP_NONE)
-        2. rplctl sends device number (0 if not required)
-        3. infod sends size of reply
-        4. rplctl stops if size is zero, otherwise displays reply
-           jump to 3
-        */
-        uint32_t dev = (uint32_t)HXlld_shift(Jobs);
-
-        send_wait(fd, &req, sizeof(unsigned char), 0);
-        PP_swab(&dev, sizeof(uint32_t));
-        send_wait(fd, &dev, sizeof(uint32_t), 0);
-
-        while(1) {
-            uint32_t reply_size = 0;
-            fd_set rd;
-
-            recv(fd, &reply_size, sizeof(uint32_t), MSG_WAITALL);
-            PP_swab(&reply_size, sizeof(uint32_t));
-            if(reply_size == 0) { break; }
-
-            while(reply_size > 0) {
-                FD_ZERO(&rd);
-                FD_SET(fd, &rd);
-                select(fd + 1, &rd, NULL, NULL, NULL);
-                if(FD_ISSET(fd, &rd)) {
-                    ssize_t eax = recv(fd, buf,
-                     min_uint(sizeof(buf), reply_size), 0);
-                    write(STDOUT_FILENO, buf, eax);
-                    reply_size -= eax;
-                }
-            }
-        }
-    }
-
-    req = IFP_NONE;
-    send_wait(fd, &req, sizeof(unsigned char), 0);
-    return;
-}
-
 //-----------------------------------------------------------------------------
 static uint32_t getdev(const char *s) {
     uint32_t major, minor;
@@ -187,6 +123,57 @@ static uint32_t getdev(const char *s) {
     return K26_MKDEV(major, minor);
 }
 
+static void read_reply(int fd) {
+    char buf[4096];
+
+    while(1) {
+        uint32_t reply_size = 0;
+        fd_set rd;
+
+        recv(fd, &reply_size, sizeof(uint32_t), MSG_WAITALL);
+        PP_swab(&reply_size, sizeof(uint32_t));
+        if(reply_size == 0) { break; }
+
+        while(reply_size > 0) {
+            FD_ZERO(&rd);
+            FD_SET(fd, &rd);
+            select(fd + 1, &rd, NULL, NULL, NULL);
+            if(FD_ISSET(fd, &rd)) {
+                ssize_t eax = recv(fd, buf,
+                 min_uint(sizeof(buf), reply_size), 0);
+                write(STDOUT_FILENO, buf, eax);
+                reply_size -= eax;
+            }
+        }
+    }
+    return;
+}
+
+static void send_int(int fd, unsigned char req, uint32_t dev) {
+    send_wait(fd, &req, sizeof(unsigned char), 0);
+    PP_swab(&dev, sizeof(uint32_t));
+    send_wait(fd, &dev, sizeof(uint32_t), 0);
+    read_reply(fd);
+    return;
+}
+
+static void send_void(int fd, unsigned char req) {
+    send_wait(fd, &req, sizeof(unsigned char), 0);
+    read_reply(fd);
+    return;
+}
+
+static void send_string(int fd, unsigned char req, const char *str) {
+    size_t size = strlen(str);
+    uint32_t szpack = size;
+    send_wait(fd, &req, sizeof(unsigned char), 0);
+    PP_swab(&szpack, sizeof(uint32_t));
+    send_wait(fd, &szpack, sizeof(uint32_t), 0);
+    send_wait(fd, str, size, 0);
+    read_reply(fd);
+    return;
+}
+
 static int unix_client(const char *port) {
     struct sockaddr_un sk;
     int fd = -1;
@@ -232,15 +219,17 @@ static int get_options(int *argc, const char ***argv) {
 
     poptContext ctx;
     const char **args;
-    int c, argk;
+    int c, argk, got = 0;
 
     ctx = poptGetContext(**argv, *argc, *argv, options_table, 0);
     while((c = poptGetNextOpt(ctx)) >= 0) {
         static const int mapping[] = {
             ['A'] = IFP_ACTIVATE,
             ['D'] = IFP_DEACTIVSES,
+            ['F'] = IFP_FLUSH,
             ['J'] = IFP_DEACTIVATE,
             ['X'] = IFP_REMOVE,
+            ['Z'] = IFP_ZERO,
         };
         switch(c) {
             case 'A':
@@ -248,24 +237,21 @@ static int get_options(int *argc, const char ***argv) {
             case 'J':
             case 'X': {
                 uint32_t dev;
+                ++got;
                 if((dev = getdev(tmp)) != 0) {
-                    HXlld_push(Jobs, (void *)mapping[c]);
-                    HXlld_push(Jobs, (void *)dev);
+                    send_int(Sockfd, mapping[c], dev);
                 }
                 free(tmp);
                 break;
             }
             case 'F':
-                HXlld_push(Jobs, (void *)IFP_FLUSH);
-                HXlld_push(Jobs, NULL);
+            case 'Z':
+                ++got;
+                send_void(Sockfd, mapping[c]);
                 break;
             case 'L':
-                HXlld_push(Jobs, (void *)(Opt.parseable ? IFP_GETINFO : IFP_GETINFO_T));
-                HXlld_push(Jobs, NULL);
-                break;
-            case 'Z':
-                HXlld_push(Jobs, (void *)IFP_ZERO);
-                HXlld_push(Jobs, NULL);
+                ++got;
+                send_int(Sockfd, IFP_GETINFO_A, 0);
                 break;
         }
     }
@@ -273,7 +259,7 @@ static int get_options(int *argc, const char ***argv) {
     if(c < -1) {
         fprintf(stderr, "%s: %s\n", poptBadOption(ctx, 0), poptStrerror(c));
         poptFreeContext(ctx);
-        return 0;
+        return -1;
     }
 
     if((args = poptGetArgs(ctx)) != NULL) {
@@ -285,7 +271,7 @@ static int get_options(int *argc, const char ***argv) {
     }
 
     poptFreeContext(ctx);
-    return 1;
+    return got;
 }
 
 //==[ End of file ]============================================================
